# CNCF Technical Advisory Groups ("TAGs")

<!-- Primary Authors: Alexis Richardson, Quinton Hoole

with contributions by TOC Contributors, November 2018 - January 2019 

v1.1

Starting an update and revisit of the TAG folder, files and document content - Dec 2023 >
-->
v1.1
## Introduction

The purpose of this document is to provide insight into the objectives and life cycle of Technical Advisory Groups (TAGs) within the CNCF. Various TAGs exist, each within in the CNCF. More information about each TAG can found on the [TAG List](cncf-tags.md) and by following the links to the TAG's respective repositories.


## Overall Purpose

Scale contributions by the CNCF technical and user community, while retaining integrity and increasing quality in support of our [mission](https://github.com/cncf/foundation/blob/main/charter.md#1-mission-of-the-cloud-native-computing-foundation).

A CNCF TAG will oversee and coordinate the interests pertaining to a logical area of needs of end users and/or projects.  Examples of such areas include security, testing, observability, storage, networking, etc.  The area overseen by a TAG is typically met by a set of CNCF projects, and may also represent a cross-cutting feature group shared by several projects (like security and observability).  TAG’s are:

* long lived groups that report to the Technical Oversight Committee
* led primarily by recognised experts in the relevant field(s), supported by other contributors

CNCF TAGs are modelled on Kubernetes SIGs.  Differences are intended to be minimal to avoid confusion - unavoidable differences are described [here](https://docs.google.com/document/d/1oSGhx5Hw7Hs_qawYB46BvRSPh0ZvFoxvHx-NWaf5Nsc/edit?usp=sharing).


## Specific Objectives

* Strengthen the project ecosystem to meet the needs of end users and project contributors.
* Identify gaps in the CNCF project portfolio.  Find and attract projects to fill these gaps.
* Educate and inform users with unbiased, effective, and practically useful information.
* Focus attention & resources on helping foster project maturity, systematically across CNCF projects.
* Clarify relationship between projects, CNCF project staff, and community volunteers.
* Engage more communities and create an on-ramp to effective TOC contribution & recognition.
* Reduce some project workload on TOC while retaining executive control & tonal integrity with this elected body.
* Avoid creating a platform for politics between vendors.
* Provide a ladder for community members to get involved with the technical oversight of CNCF projects. As part of this, TAGs are expected to actively nurture diverse participation.

## Responsibilities & Empowerment of TAGs

CNCF TAGs should, under guidance from the TOC, provide high-quality technical expertise, unbiased information and proactive leadership within their domain.  The TOC makes use of this input to act as an informed and effective executive board to select and promote appropriate CNCF projects and practices, and to disseminate high quality information to end users and the cloud native community in general.  

TAGs explicitly have no direct authority over CNCF projects. In particular, the creation of CNCF TAG’s does not change the existing, successfully practiced [charter](https://github.com/cncf/foundation/blob/main/charter.md) goal that "Projects.. will be ‘lightly’ subject to the Technical Oversight Committee".

### Project Engagement

Projects themselves may engage with the TAGs:

* when exploring engagement within the cloud native ecosystem
* prior to submitting their application for CNCF consideration
* after submitting their application for CNCF consideration
* for guidance in moving levels within the CNCF
* upon direction by the CNCF Technical Oversight Committee (TOC) or Linux Foundation staff

Project engagement within the Cloud Native ecosystem is a vital way to raise project visibility as well as gain insights from technical leaders in the CNCF community for project maintainers.

As the Technical Advisory Groups (TAGs) are an entry point for many projects to engage with the Cloud Native ecosystem, TAGs are encouraged to guide projects to join the TAGs, Working Groups, collaborate with other projects for development of specifications or promote interoperability as well as explore various avenues to further engage with the greater community.

### Technical Leadership
The TAGs should strive to present the TOC with easily understandable and votable "propositions", each of which is supported by clear written evidence.  A proposition may be “to approve this project for incubation based on ..." or “to approve this landscape document under its clear goals and the evidence that it achieves them”.  It is of utmost importance that the information and proposals provided to the TOC by TAGs be highly accurate and unbiased, driven by the goal to improve the CNCF as a whole, rather than benefit one project or company over another.  We believe that the rising tide lifts all boats, and that is our goal.

Key ideas here:

* The TOC is the arbiter & editor and may always intervene and overrule. 
* The TAGs are the productive talent, and respected as such.

TAGs may choose to spawn focused and time-limited working groups to achieve some of their responsibilities (for example, to produce a specific educational white paper, or portfolio gap analysis report).  Working groups should have a clearly documented charter, timeline (typically a few quarters at most), and set of deliverables. Once the timeline has elapsed, or the deliverables delivered, the working group dissolves, or is explicitly re-chartered.

### Specific TAG Responsibilities 

#### Project Handling:

* Conduct [Domain Technical Reviews](resources/toc-supporting-guides/project-reviews.md) of projects to inform the TOC for Sandbox Review or as requested for projects moving levels (by projects, or by TOC members if deemed beneficial)
* Understand and document a high level roadmap of projects within this space, including CNCF and non-CNCF projects. Identify gaps in project landscape.
* For projects that fall within the CNCF, perform health checks.
* Perform discovery of and outreach to candidate projects
* Every CNCF project will be assigned to at least one suitable TAG by the TOC.

#### End User Education (Outbound Communication)

* Provide up-to-date, high quality, unbiased and easy-to-consume material to help end users to understand and effectively adopt cloud-native technologies and practises within the TAG’s area, for example:
  * White papers, presentations, videos, or other forms of training clarifying terminology, comparisons of different approaches, available projects or products, common or recommended practises, trends, illustrative successes and failures, etc.
  * As far as possible, information should be based on research and fact gathering, rather than pure marketing or speculation.  

#### End User Input Gathering (Inbound Communication)

* Gather useful end user input and feedback regarding expectations, pain points, primary use cases etc.
* Compile this into easily consumable reports and/or presentations to assist projects with feature design, prioritization, UX etc.

#### Community Enablement

* TAGs are open organizations with meetings, meeting agendas and notes, mailing lists, and other communications in the open
* The mailing list, TAG meeting calendar, and other communication documents of the TAG will be openly published and maintained

#### As Trusted Expert Advisors to the TOC

* Perform [Domain Technical Reviews](resources/toc-supporting-guides/project-reviews.md) on upcoming Sandbox projects for review as well as projects moving levels as requested by the TOC, following the project reviews guidance.
* Be involved with, or periodically check in with projects in their area, and advise TOC on health, status and proposed actions (if any) as necessary or on request.

#### TAG Charter:

* This is formally reviewed annually, and approved by the TOC.  The charter must clearly articulate:
  * what is in and out of scope of the TAG,
  * whether and how it overlaps and interfaces with other CNCF TAG’s or other relevant groups, and 
  * how it operates and is governed, and specifically whether and how it deviates from standard TAG operating guidelines provided by the TOC.  Deviation from these guidelines is discouraged, unless there are good and well-documented reasons for such divergence, approved by the TOC. 

See [Example Responsibilities of a CNCF TAG](https://docs.google.com/document/d/1L9dJl5aBFnN5KEf82J689FY0UtnUawnt9ooCq8SkO_w/edit?usp=sharing).

## Operating Model

Important: Each TAG is supported by a named member of the CNCF executive staff who is accountable for liaison with the CNCF Executive Director, plus communication and performance of the TAG, with quarterly and annual reporting to Governing Board & TOC.

As a starting point let’s be inspired by CNCF OSS Projects and by K8s SIGs.  That means minimal viable governance and community-based organisation.

### TAG Formation, Leadership and Membership Composition

1. The Technical Oversight Committee (TOC) takes responsibility for the establishment of TAGs. In order to ensure the viability, sustainability, and relevance of a TAG, the TOC requires proposals for TAGs to first begin within an existing TAG as a working group. When a working group reaches sufficient momentum, interest, and growth that aligns with cloud native goals and objectives, has alignment with several cloud native projects, and shows continued execution in alignment with their charter, the TAG and Working Group Leadership may engage their TOC Liaisons to determine if the Working Group is eligible to be reconsidered as a TAG. This process may take a few years as these specific domains evolve and mature in similar fashion to cloud native projects evolution and maturity. The TOC may then vote to instantiate a new TAG. The TOC wishes to have the smallest viable number of TAGs, and for all of them to be highly effective (as opposed to a "TAG sprawl" with large numbers of relatively ineffective TAGS).
2. TAG has three co-chairs, recognized as experts in that area, and for their ability to co-lead the TAG to produce the required unbiased outputs.
3. TAG has at least one TOC liaison who is a voting member of the TOC acting as an additional non-executive chair on occasions when TOC input is deemed necessary by the TOC or the TAG chairs for TAG decision making.
4. TAG has multiple tech leads who are recognized as (1) experts in the TAG area, (2) leaders of projects in the TAG’s area (3) demonstrating the ability to provide the balanced technical leadership required to produce the required unbiased outputs of the TAG. The reason for having separate chair and tech lead roles is to allow responsibility for primarily administrative functions to be separated from deep technical functions and associated time commitments and skill sets.  Where appropriate, an individual may perform both roles as shown in [TAG member roles](/tags/README.md#tag-member-roles).
5. Participant diversity is strongly encouraged within TAGs, and TAG chairs are expected to actively encourage a diverse range of community members to take up named roles (see point 7). 
6. A variety of perspectives is strongly encouraged within TAGs.  To this end, a supermajority (⅔ or more) of chairs or a supermajority of tech leads from a single group of related companies, market segment, etc will be actively discouraged by the TOC.
7. TAG members are self-declared, so that some TAG work is done by volunteers from the community.  To recognise members who make sustained  and valuable contributions to a TAG over time, TAG-defined and assigned roles may be created (e.g. scribe, training or documentation coordinator etc).  TAG’s should document what these roles and responsibilities are, and who performs them, and have them approved by TAG leads. TAGs roles should have active mentoring and shadowing programs to encourage sustainability of the TAG.

### TAG Member Roles

#### Chair

* Three chairs where the active chair rotates each week/fortnight/month.
* Primarily performs administrative functions including collecting and compiling topics for the (bi)weekly agenda, chairing the meeting, ensuring that quality meeting minutes are published, and follow-up actions tracked and resolved.
* A chair role may be held and performed by a tech lead, in cases where one person has the time and ability to perform both roles to the satisfaction of the TOC and TAG members.

#### Tech Lead

* Leads projects in the TAG’s area.
* Has the time and ability to perform deep technical dives on projects.   Projects may include formal CNCF projects or other projects in the area covered by the TAG.

#### Working Group Lead
* Lead the activities of the working group under the TAG.
* Have the domain knowledge to lead the working group.
* Approved by supermajority of the chairs, the TAG liaison, and the TOC liaisons to the TAG.

#### Other named roles

* Named and defined by the TAG (e.g. scribe, PR lead, docs/training lead, etc)
* Approved by supermajority of the chairs.

#### Other members

* Self-declared
* May either have no explicit roles or responsibilities, or formally assigned roles (see above).
* May not create the impression that they have any authority or formal responsibilities in the TAG other than assigned roles.

### TOC Liaisons

TOC is the technical governing body of the CNCF. It’s important that we ensure a good flow of governance information and feedback loops to and from the TAGs. With several TAGs, it can be hard to connect for community wide consensus, therefore, each group is assigned at least one TOC liaison.

TAG leads may call on liaisons to act as a point of contact from the TOC, be an advisor for governance or community health matters, or help the TAG prioritize what they’re doing to be in line with TOC needs.

Liaisons do not make decisions for the TAG.

Liaisons are assigned based on their expertise, interest, and specialized experience a given TAG may need at the time of assignment (adjustments can be made, if needed).
The TOC relies on the Chairs, Leads, activities, members, and other TOC members to ascertain those needs through their interactions and observations.
For instance, a TOC member with a strong background in Networking and Green Computing was previously assigned to TAG Network, they may rotate to TAG Environmental Sustainability.
Alternatively, a TAG may need a TOC Liaison with specialized experience in growing contributors and sustaining communities, however their expertise is in App Delivery.
In this instance they may rotate from App Delivery to the TAG that needs to grow their own contributors or sustain their member pool and activities - not necessarily rotating to TAG Contributor Strategy.

#### Responsibilities

- Streamline communications between TOC and TAG chairs, and helps TAG to set priorities.
- Communicate activities of the TAG and it's WGs to TOC beyond the public monthly updates.
- Provide updates regarding TAG activities and potential areas of improvement to the TOC at least once a quarter.
- Support and guide growth and development of the TAG and it's WGs.
- Approve creation of WGs for TAGs they are a liaison for.
- Perform the final review and approval of technical content produced by the TAG.
- Attend TAG meetings, as needed/requested.

#### Expectations

- Check-ins with their TAGs at least once a month (or more frequently as needed). This could mean a group DM with Chairs, a visit to a TAG meeting, or some other connection. Get a pulse on health of the following:
  - Contributor base
    - How are the chairs, technical leads and contributors doing?
    - Are newcomers able to efficiently join and contribute?
  - Operational Health
    - Transparency of meetings and decision processes (eg: published meeting agendas, minutes, and video recordings)
    - Communication best practices for shared status and decision making across a TAG and related WGs.
    - Provide consultation on how to improve operational health.
  - Gap Analysis
    - Where is the TAG now and where do they want/need to be?
  - Sustainability
    - What is the TAG doing to ensure contributions are sustainable?
    - Is the TAG leadership healthy?
    - Are there mentorship opportunities for TAG members to grow into leadership roles?
    - What does the TAG need in order to keep things going?
    - Are there areas of concern?
  - Project alignment and maturity within the domain
    - Are projects engaged with the TAG?
    - Is the TAG supporting technical maturity of the projects aligned in their domain?
    - Is the TAG and the TOC aligned on the current maturity of all projects within the TAG’s domain?
- Keep the TOC in the loop and keep track of trackable (i.e. not sensitive) work in public issues.
- Communicate with the TAGs on important business that happens at the TOC in order for the TOC to achieve a pulse on the work they are doing and acknowledge governance changes.
    - For example, if the TOC is crafting a new policy, check in with the chairs to see how it would fare with their groups.
- Receive TOC liaison approval
  - For creation of Working Groups - TOC liaisons verify WGs are appropriately scoped, in alignment with the charter, and capable of achieving their defined deliverables.
  - For publication of any technical content - TOC liaisons perform the final review and approval of technical content produced by the TAG (e.g. whitepapers). This ensures the liaisons are aware of the deliverables as well as identify any gaps, missed clarity, and bring the content into alignment. This review should occur after all public comments are adjudicated and the TAG co-chairs have reviewed and concurred.

### Elections

* For new TAGs, the TOC nominates Chairs. For existing TAGs, the TOC relies on the TAG leadership to present candidates to the TOC.
* Chairs are assigned following a 2/3 majority vote of the TOC
* Terms last for 2 years but staggered such that at least 1 of the chairs is able to maintain continuity
* The TOC and Chairs nominate Tech leads
* Tech leads are assigned following a 2/3 majority vote of the TOC and a 2/3 majority vote of TAG Chairs
* TAG Chairs and Tech Leads may be unassigned from the TAG at any time following a 2/3 majority vote of the TOC

### Governance

* All TAGs inherit and follow the CNCF TOC Operating Principles.
* TAGs must have a documented governance process that encourages community participation and clear guidelines to avoid biased decision-making.  
  * NOTE: aim here is to align with "minimal viable" model of the CNCF projects, and only have such governance as is needed, not anything too burdensome
* They may grow a set of practices over time in the same way as an OSS Project, provided this is consistent with CNCF Operating Principles. 
* As with CNCF Projects all exceptions and disputes are handled by TOC with CNCF Staff help

### Budget & Resource

* No formal systematic budget at this time, other than commitment of CNCF executive staff to provide named person as liaison point.
* Just as CNCF Projects may have "help" offered by CNCF and may ask for things via the [ServiceDesk](https://github.com/cncf/servicedesk), the TAGs may do this.

## Retirement

* In the event that a TAG is unable to regularly establish quorum, or fulfill the responsibilities and/or regularly report to the TOC, the TOC will:
    * Consider retiring the TAG after 3 months
    * Must retire the TAG after 6 months

* The TOC may, by means of a 2/3 majority vote, declare "no confidence" in the TAG.  In this event, the TOC may then vote to retire or reconstitute the TAG.
