# Governance Remediation Escalation 

This governance remediation process is for cases where CNCF projects are either not practicing their governance as documented or the project is stuck in a circumstance that its current governance does not cover. As projects evolve over time, we expect projects to evolve their governance as well, the key being the documentation gets updated and reflects current practice(s) quickly before there is a crisis. For projects that would like help adding to or updating their governance, please consider opening a [governance review issue](https://github.com/cncf/tag-contributor-strategy/issues/new?assignees=jberkus%2Cgeekygirldawn&labels=wg%2Fgovernance&projects=&template=governance-review-request.yaml&title=%5BGovernance+Review%5D%3A+PROJECT+NAME) with TAG Contributor Strategy.

## Importance of clear governance documentation

Throughout these processes it is important to maintain clear documentation of all steps taken, decisions made, and actions implemented and provide regular updates to the community and stakeholders about the process and its outcomes. This governance remediation process ensures that CNCF projects maintain effective governance structures, resolving conflicts and gaps promptly while fostering a collaborative and transparent community environment.

## Pre-escalation expectations

Project members, stakeholders, and community members should first attempt to resolve governance issues through normal project discussion and consensus. The Remediation process exists for when normal, in-project resolution fails, either because of a difference of opinion, or because the project lacks the tools to make sufficient changes themselves.

## Governance Remediation

### Objectives:

#### Establish Missing Governance 

_Assist projects in developing governance structures to handle situations they are currently unequipped to manage._

It's normal that as a project grows, it will encounter circumstances that weren't previously covered by the governance documentation, such that there is a lack of a clear path forward. This "missing governance" can usually be remediated amicably and internally within the project, by discussing proposed governance changes, and the maintainers agreeing to adopt changes. In these cases, missing governance is an opportunity for improvement. 

#### Correct Non-Practiced Governance 

_Ensure projects adhere to their documented governance._

Non-practiced governance can either be someone willfully or inadvertently ignoring stated governance. Alternatively, occasions may arise where the defined governance _cannot_ be practiced as the project may be lacking people in a role to execute governance processes.

When a governance conflict or unaccounted situation arises, and the project cannot reach a consensus to develop a path forward. The TOC will work with project stakeholders, TAG Contributor Strategy, the Code of Conduct Committee, and CNCF Staff to create a public remediation plan for the project.

This plan could involve changes to the written governance, removal of specific project leaders, or whatever other measures are required to restore the project to a state where it is following its own governance rules, and that governance is in accord with CNCF principles. Should there be no way to restore the project to community operation, the TOC can consider archiving the project.

In rare cases, governance failure may be accompanied by possible CoC violations by individual project leaders. Should this happen, the assigned team will engage the CNCF Code of Conduct Committee and the two teams will coordinate their evaluation.

### Out of Scope:

**Conduct Issues**: Handled by the CNCF Code of Conduct [processes](https://github.com/cncf/foundation/blob/main/code-of-conduct.md#reporting). This includes behavior by project leaders which is a violation of the CNCF CoC; such behavior should be reported to the CoCC and not handled through Governance Remediation.

**Leadership Issues**: Behavioral issues with individual project leaders may be addressed by contacting the TOC regarding violations of the [Technical Leadership Principles](https://github.com/cncf/toc/blob/main/PRINCIPLES.md#technical-leadership-principles).

**Technical Disagreements**: Projects should use or establish governance to resolve such disputes independently.

**Lifecycle Processes**: Elements already covered in the project's lifecycle review.

## Reporting Governance Remediation

**Public Reports: **In most cases, project governance issues will be reported publicly to the TOC by filing [an Issue in the TOC repository](https://github.com/cncf/toc/issues) of type Governance. Should a public issue be reported in some other venue (the project's issue tracker, mailing list, etc.) an issue in the TOC repo should be created for tracking and assignment.

**Anonymous Reports**: Should the reporter not wish their name to be known, they should email cncf-private-toc@lists.cncf.io.  The TOC team will then create an issue in the TOC repo tracking the remediation. The identity of the anonymous reporter will be known only to the TOC team.

**Confidential Reports**: Should there be risks around the details of the remediation request being public, the reporter will email cncf-private-toc@lists.cncf.io.  The TOC and the assessment team will then use privately shared documents to track the status of the remediation request.

## Governance Remediation Process

### Roles

* TAG Contributor Strategy: For missing governance the assignee will lead, consulting and informing with the assigned TOC member as needed. For non-practiced governance, they act as an advisor to the TOC
* TOC member assignee(s): serves an oversight/informational function that the governance assignee can leverage as needed to impress upon the project the needed changes or leads the discussion with the project in the case of non-practiced governance
* Code of Conduct Committee: if a failure of governance coincides with CoC violations by one or more project leaders, the CoCC will coordinate with the assessment team.

### Missing Governance

1. Assessment Team Formation: The TOC will assign one TAG Contributor Strategy assignee and one TOC member.

2. Investigation: The assessment team will Interview the reporter, project maintainers, and other relevant contributors to determine if the governance issue is valid and assess the project's ability to resolve it internally.

3. Recommendation Development: In collaboration with the assessment team, the project will draft new governance policies to address gaps. This will include an actionable plan for the project to follow, with specific milestones and deadlines.

4. Implementation and Monitoring: The assessment team will regularly check in on the project's progress towards defined milestones, provide additional support as needed, and provide a public progress report at 3 and 6 month milestones.

5. Resolution or Escalation: If the project meets the milestones, the issue can be resolved. If not, it should be transferred to the Non-Practiced Governance process.

### Governance Remediation Process: Non-Practiced Governance

1. Triage: The TOC will do an initial triage of the issue and assign a lead member and a supporting member to guide the process.

2. Assessment Team Formation: The two TOC members will work with a member of [Governance WG](https://contribute.cncf.io/about/governance/) to form an assessment team.

3. Investigation: The assessment team will Interview the reporter, project maintainers, and other relevant contributors to determine if the governance is not being practiced and assess the project's ability to resolve it internally. This may include attendance by the project’s leadership team to a private TOC meeting to discuss the issue more in depth.

4. Recommendation Development: The assessment team will determine where the project is not following its documented governance and work with the project to create recommendations on how to get back on track. This will include an actionable plan for the project to follow, with specific milestones and deadlines. These recommendations will then be shared with the whole TOC for further feedback and commentary.

5. Implementation and Monitoring: The assessment team will regularly check in on the project's progress towards defined milestones, provide additional support as needed, and provide a public progress report at 3 and 6 month milestones.
