# Welcome to the TOC!

We are so glad you are here. You have been elected to the Technical Oversight Committee, the body responsible for reviewing, accepting/rejecting, and overseeing projects in the cloud native ecosystem.  This onboarding document and its various files and links are here to help get you up to speed and set you up for success as a TOC member. If you haven't already done so, now is a great time to familiarize yourself with the [Definition of Cloud Native](../DEFINITION.md).

#### About this document

This onboarding document is designed to provide some introductory information to new TOC members. It contains limited information to set expectations and provide a TOC member with generalities about the role and work we do. For specific instructions, guidance, templates, etc. please refer to the various files and content in the [Operations Directory](README.md).

## The Charter

The role of the TOC is defined by the [CNCF Charter](https://github.com/cncf/foundation/blob/main/charter.md). Please take a few moments to familiarize yourself with the charter.  The TOC is mandated in section 6(a), however there are many parts of the charter that are relevant to the execution of our work.

Within the scope of the Charter, the TOC is responsible for defining how we achieve that mandate. That means we agree on the structure, processes, and tasks. Everything that follows represents the current way the TOC has been working. It is not set in stone, as it is expected that the TOC will periodically alter, adjust, and change how we conduct our work in response to shifts in the landscape and community as well as proactively, to not only keep pace, but lean into technical innovation for a sustainable cloud native future.

## Accesses, Openness, and Transparency

As TOC members, we manage our repository with the support of CNCF Staff. You will be added to the toc-members group and granted appropriate permissions on the repository so you may create, review, and merge pull requests in accordance with our [PR processes](../toc-decision-process.md). We bear the responsibility of managing our repository equally among the TOC and expect all TOC members to support the collective group in managing the repo. 

The TOC maintains both public and private communication mediums. In accordance with the [CNCF values](https://github.com/cncf/foundation/blob/main/charter.md#3-values), we strive for openness and transparency - our technical work must be available to all. Our processes and decisions shall be transparent, visible, and discoverable. The nature of some TOC discussions may be sensitive as they impact individuals, projects, and organizations. Therefore TOC members are expected to use their best judgement in determining if a discussion should be held in private to allow the TOC members to speak their mind or raise doubts fully and frankly without the fear of being misunderstood or revealing their ignorance in front of a wider audience.  Once consensus is reached, the rationale and reasoning will be summarized, well documented, and a decision may be put forward for the community publicly, either as informative or for public review and finalization through voting. 

You will be added to both public and private slacks, mailing lists, and receive calendar invitations for both public and private meetings. Reach out to CNCF Staff or the TOC Chair if you believe you are missing any invites or lists. Your inclusion on these lists as a member of the TOC enables you to field questions and concerns from community members and redirect as needed. 

## Attendance, Availability, and Involvement

The expected time commitment of a TOC member is anywhere from **10-25% of their monthly regular work time** and will vary month to month with surges and slow downs throughout their term. TOC members should also anticipate travel to conferences and offsites at least twice a year, with participation and attendance at various meetings during the CNCF flagship conference: KubeCon+CloudNativeCon (KCCN).

The TOC has four (4) hour-long meetings per month held on Tuesdays at 0800 PST/1100 EST/1700 CET.  The first meeting of each month are [Technical Advisory Group (TAG)](../tags/README.md#cncf-technical-advisory-groups-tags) updates and discussion. As a [Liaison](../../tags/cncf-tags.md#toc-liaisons) to one or more TAGs, the content presented and discussed by the TAG should be information you are already aware of. For TAGs that you are not a liaison for, this is your opportunity to become aware of the broader work being done within specialty technical domains and verticals.  It allows you to connect various TAGs and projects together, promoting interoperability for the benefit of the technical community and its adopters.

The second and fourth meetings are generally private meetings. During these meetings, the TOC meets to review issues on the repo, conduct vote parties (to ensure outstanding votes are completed), and discuss any open issues, activities, tasks, and other efforts. Periodically, we may invite the Maintainers, projects, or other groups to join us so we may connect with them, understand any challenges and successes they encounter, and take feedback to improve our engagements.

The third meeting is public and will usually cover public discussion on improvements and activities the TOC is pursuing or community-led topics where the TOC has oversight or is needed to facilitate neutral consensus.

TOC members rotate meeting facilitation as often as we can to ensure the TOC Chair is not always doing it, they already have a lot of tasks, see [TOC Chair](toc-chair-vicechair.md) for more detail. TOC members should review the slack discussion and meeting notes prior to the meeting to ensure they are apprised of the intent and focus of the meeting. Commonly, the TOC Chair will work with CNCF staff to ensure meeting agendas and topics are provided to the TOC well in advance of a meeting. The meeting topic calendar remains flexible to ensure more pressing issues are discussed and resolved quickly. TOC members are welcome to bring up topics for discussion or addition to the agenda and meeting calendar by requesting it in the TOC channel. Commonly, CNCF Staff and the Chair are already monitoring the channel, repo, and mailing list for topics that need to be addressed and may already have them included.

### Expected functions of a TOC member

In the course of performing our duties, TOC members are expected to:

* Lead or meaningfully contribute to at least 1 Project Due Diligence per year that is brought to TOC internal review status. The TOC Chair or Vice Chair (with support of Staff) will determine if the DD brought forward meets the TOC's DD criteria above and beyond the technical review by the TOC.
    * Some Due Diligence efforts may result in the TOC internal review stating the project is not ready after adopter interviews, these are still considered as meeting this expectation as determined by the Chair or Vice Chair. 
    * Lightweight reviews/triage of DD applications, while also required, do not meet the expectations of leading or participating in a Project DD.
* Be available to field questions and concerns from community members and redirect as needed
    *  While every TOC member is expected to do this, there may be occasions or instances where there is a lot of back and forth and the Chair or Vice Chair will coordinate and provide the response as the central contact
* Foster discussion - ask questions, even obvious ones to move the conversation forward or to closure
* Bring project and group conflicts that escalate to the TOC level to closure, either through elevating awareness within the TOC for discussions and consensus, direct resolution while keeping the TOC informed, or escalation to the Chair and Vice Chair as appropriate
* Participate in meetings, presentations, etc. as needed at KCCN or other events

#### Competencies

In addition to the expectations defined Composition 6(b)ii and Criteria 6(d) for nomination in the CNCF Charter, TOC members should be prepared to actively employ and mature the following competencies:

* Leadership (ex. transformational, democratic, servant, and visionary are leadership styles that may be used depending on the situation)
* Community management for guiding TAGs and projects
* Deep understanding of cloud native technologies and viability for use
* Communication (empathy, inclusivity, effectiveness, action-oriented)
* Time and task management
* Accountable autonomy
* Context conveyance
* Critical thinking and advanced problem-solving (ex. causal analysis, troubleshooting, logical reasoning)
* Conflict resolution
* Improvement-oriented mindset
* Data-driven decision making 
* Open-mindedness
* Market awareness (technical and economic trends and impacts)
* Software engineering methodologies and concepts
* Systems and strategic thinking
* Problem solving through automation-driven approaches
* Active listening

### Events and additional meetings

#### Governing Board

The TOC attends Governing Board meetings periodically when invited, our participation is typically limited to 1 hr ([except for the TOC Chair](toc-chair.md#Governing-board-meetings)). The most common of these is the Strategy Meeting typically held during KubeCon+CloudNativeCon (KCCN). As the overseeing technical body, it is anticipated the TOC guides any technical strategy discussions. The TOC collectively crafts the messaging and structure for this in response to interest areas expressed by the Governing Board or by technical trends in broader industry and within the ecosystem.

#### KubeCon+CloudNativeCon (KCCN)

At KCCN, the TOC has three additional engagements; TOC Keynote, TOC Panel, and TOC/TAG Meeting.  The TOC keynote is about 15 minutes in length and is delivered by one or more TOC members and may include guests. Common topics are:
* introducing what the TOC is
* conveying changes to the ecosystem/landscape since the last KCCN
* community oriented topics

The TOC Panel is a panel of the TOC members present at the conference and functions similar to an ask-me-anything session. The CNCF CTO initiates the panel by providing an overview to attendees of who the TOC is and what we do. They will often initiate the first few questions to the panel and facilitate attendee questions to the TOC.

The TOC and TAG meeting may be an open (conference attendees may observe) or closed meeting, depending on the topics. This meeting focuses heavily on the TAGs, their health, and their work. TOC Liaisons are expected to make time to meet with their TAGs as part of that meeting.

There are additional social events at KCCN that TOC members are invited to as a body or as individuals, attendance at these is purely optional.

#### TOC Offsite

The TOC offsite is a relatively new event, happening at least once per year lasting about 1-2 days plus travel. It is focused time, ideally face-to-face but virtual is also an option, for the TOC to come together and perform strategic planning and operational health development of the TOC, community, TAGs, and projects. Many initiatives the TOC has had success in executing are the result of these off-sites and thus highlights their importance.

### Preparedness and communicating availability

TOC members, in accordance with the Charter Section 6(f)iii, are expected to attend and be prepared in-advance of each meeting so that the conversation may be productive and decisions may be reached in a timely manner.  TOC members are **not full-time employees of the CNCF**, they are individuals elected to serve the community - many with full-time jobs for which their employers support their activities - *for the benefit of cloud native and open source*.  As such, we understand that incidents, pressing issues, and other primary work priorities may occur from time-to-time, particularly as many TOC members hold significant technical leadership roles within their organizations, that may inhibit their ability to make every meeting all the time. Further, we are human with personal lives and responsibilities that may temporarily limit our ability to attend and participate in these meetings, e.g. vacations, personal matters, etc. It is expected that every TOC member *will communicate their absence or limited availability* in advance (when possible), but endeavor to be present and prepared for these meetings. 

In addition to these meetings, TOC members are expected to meet synchronously or asynchronously with the TAGs for which they liaison, occasionally attending TAG meetings.

### Inactivity of a TOC member

If, for any reason, a TOC member is continuously absent without notice, see 6(f)iii, their voting privileges are *suspended* until such time as their consecutive attendance at two meetings. Due to the nature of our work, the inability of a TOC member to cast an *informed* vote (as occurs by meeting attendance *and* involvement in regular engagement as detailed above) penalizes the entire TOC's ability to regularly achieve quorum.  If after the 4th absence the TOC is not notified, the TOC Chair with support of CNCF Staff will make an attempt to address the absence with the TOC member. 

If a TOC member is unable to fulfill the committed work and is unable to support due diligence to TOC internal review, that TOC member will be considered inactive, and the TOC Chair, with support of CNCF Staff will attempt to address the lack of progress with the TOC member.

##### Removal

If the Chair and Staff are unsuccessful, the Chair will bring the issue of absence or lack of progress to TOC vote in accordance with 6(f)ii at the next private meeting and provide notice to the Governing Board of the outcome.


#### Stepping down

If, for whatever reason, a TOC member feels they cannot sustain the commitment required of the role, they may proactively notify the TOC Chair or CNCF Staff of their decision to step down.  While uncommon, this is entirely reasonable to occur, particularly as lives and careers grow and shift. We thank each TOC member for their time of service to the community, however long or brief it may be.

### Communicating effectively for consensus

One of the most critical aspects of the TOC is our ability to facilitate effective communication, particular among ourselves. The TOC holds many discussions over Slack. Members do their best to summarize content and may use emojis to indicate types of information for quick review or action. An effective tactic that is being normalized is:
* `:information_source:` (light blue square with an 'i' in it) This emoji is used to indicate the content is for information only and no action is needed.
* `:reminder_ribbon:` (yellow ribbon shaped like an open bottom figure eight) This emoji is used to indicate the content was previously provided and is providing a courtesy reminder. It usually indicates some task or action needs completed if you haven't already done so. It is also used to remind members of upcoming items such as topics on an upcoming Agenda.
* `:exclamation:` (red exclamation point) This emoji is used to indicate an action is needed by the TOC, either for an individual or for the group.

Emojis can be used individually or with other emojis, this usually happens to indicate a reminder has an action with it or a reminder is informational but an update may make it more topical. It is common practice to **bold** the topic immediately following the emoji for longer updates, such as summarization of a meeting.

We operate on lazy consensus, if a TOC member is not participating in a discussion, it is their responsibility to keep themselves informed of the outcome and impetus of these discussion to guide future decision making as part of their participation in the TOC, the use of emojis and summarization of topics is intended to facilitate this in the most effective manner for a group of time-constrained individuals.

## Reviewing projects

### Sandbox

Another element of the TOC's work is to review project applications for Sandbox level. The TOC typically gets through 10-12 applications in a single session. TOC members are expected to review the "Upcoming" queue of the [Sandbox Application Board](https://github.com/orgs/cncf/projects/14), pose any clarifying or expanding questions on the application to the project in advance of the meeting in order to ensuring enough information is present to make a decision from. TOC members are also expected to speak about their perspective or opinion on including the project in CNCF. If the application is unclear on how the project fits into the ecosystem or if you feel we need a recommendation from the TAG and one has not yet been given, add a comment requesting the project present to the TAG and request a recommendation from the TAG on application by adding the GitHub handles of the TAG co-chairs and requesting it.

### Due Diligence (Incubating and Graduating)

The primary work a TOC member engages is in conducting the due diligence of cloud native projects as they advance in their maturity, this process is commonly referred to the moving levels or maturation process. After a TOC member steps forward to sponsor a project, it is anticipated that a kick-off meeting is held to set expectations for what the process will be like, provide rough timelines, collect additional information and allow the project to ask any questions.  This kick-off is commonly conducted as a virtual meeting, not lasting more than an hour. Additionally, TOC members should anticipate a few hours of concentrated review, analyzing the project, its repository(ies), and clarifying the content of the due diligence. Additional time is expected to be spent conducting and summarizing Adopter interviews to ascertain the current usage of the project within the industry.

### Health and alignment

The TOC may choose to review a project as a result of a health indicator, project event, or other occasion. These reviews are typically specific to the occasion that warranted them but usually involves, researching the problem or indicator, understanding if the metric or occasion is accurate in its depiction and warrants attention, action, or other next step.

When a health or alignment concern is elevated to the TOC, TOC members should adhere to the [project health review guide](project-health-review.md).

## Voting

The TOC primarily votes in one of two mechanisms, gitvote or on the mailing list. It is our preference that voting occur with [gitvote](https://github.com/cncf/gitvote) in the repository as this provides better transparency and is easier to verify and track vote execution. Voting conducted over the mailing list is performed by responding with a +1 Binding (+1B), or -1 Binding (-1). TOC members may abstain if they have a conflict of interest. Indication of abstention from voting for any other reason is discouraged and should not be confused with not yet voting as indication of abstention is active, not passive, and results in no vote being rendered by that individual. In the absence of information, we cannot make an informed decision, so it is imperative that we ask questions openly and provide answers with clarity for our peer TOC members, particularly on topics that may not be their strength. A vote passes with two-thirds vote of votes cast based on the charter (see [6(c)(viii)](https://www.cncf.io/about/charter)). In the rare event that an issue requiring a vote is sensitive, the TOC will leverage the private mailing list to conduct the vote with the resulting decision made public.

### Impartiality and Conflicts of Interest

The CNCF Charter section [6(d)iv.](https://github.com/cncf/foundation/blob/4c7848a2ddf624ef7e77ddc988a9c9abf679513b/charter.md?plain=1#L223) describes the expectation and criteria for TOC members to operate neutrally. This requires us to ensure our actions and decisions as TOC members are impartial and — to the extent possible — unassailable. 

As a result, our voting structure and processes are designed to significantly reduce the amount of bias that may occur and influence the direction of our community, projects, and groups. In keeping with our transparency objectives, TOC members with hard conflicts of interest are expected to abstain from voting and cite a conflict is present; not voting without comment is unacceptable. 

For more information on Conflicts of Interest, please refer to the [TOC DD Guide's section on conflicts of interest](dd-toc-guide.md#conflicts-of-interest). We expect TOC members to use their best judgement in determining if a conflict is present, if you are unsure please ask!

#### Structures to reduce bias

The sandbox application process identifies “Project Champions." In the past, any TOC member that is associated with the project (either working on or provided guidance to) is commonly identified as a Champion. However, this is not a hard requirement and up to the applicant to express this value. 

The sandbox voting process removes any single member’s ability to dictate preference or favoritism on accepting a project. However, discussions for voting consideration (whether a project should move to a vote or not) can be highly influential and very informative. TOC members familiar with a project or technical area can and should certainly speak to their experience and insights, while being mindful of how they're presenting information to avoid the appearance of bias. If it cannot be avoided, be sure to recuse yourself.

For incubating and graduation, due to the limited number of TOC members available to sponsor the application for moving levels, Conflicts of interest on incubation and graduation are to be provided. While the identity and affiliation of the sponsor are ultimately irrelevant, a single sponsor could make assertions about the project without oversight up until the TOC internal review (more below). Regardless, if a perceived or real conflict is present, a secondary unconflicted TOC member is also assigned as sponsor. 

TOC members are expected to review one-another’s work for moving levels during the internal 1-week review, as it assists in reducing biases regardless of what conflicts are perceived or claimed. It is also an opportunity to understand the project more in depth, identify any areas of that may have been missed, and to allow the TOC to make an informed voting decision.

Ultimately voting is the final arbitration to reduce bias as a result of conflict of interest and when preceded by the public comment period, it is anticipated sufficient public, non-TOC reviewers have evaluated the content presented to the community for public record and informed decision making.

## Lobbying and Direct Messages

Historically, the TOC has had a problem with “lobbying”, e.g. a project approaching individual TOC members for feedback / comment, often moving on to another TOC member if they didn’t get the answer they wanted. This was particularly true for the old Sandbox model and was a key reason for changing from sponsorship to a voting model. With this in mind, TOC members should disclose to the rest of the TOC if they have been approached by a project or individual within the community or have any hard or soft conflicts of interest. It is acceptable and expected that projects or community members will reach out to the TOC for advice, however to maintain consistency in expectations, *all TOC members should be made aware of those discussions, with the opportunity to provide additional information or recommendations*. This ensures the TOC maintains a strong, clear voice to our technical community, consistent with our processes, [values](https://github.com/cncf/foundation/blob/main/charter.md#3-values), [principles](../PRINCIPLES.md), and the Charter.

As the TOC continues to work through updating our documentation on the repo, older references to "seeking sponsorship" may result in projects reaching out to TOC members directly to "sponsor" their project for moving levels.  When this occurs, kindly direct them to the Moving Levels resources which details how projects are recommended for promotion to the next level.  The TOC strives to pick up projects from the backlog in the order which they are recommended for promotion, while balancing alignment of TOC member expertise, liaisoning assignment, and availability of our members. 

## Public persona and engagement

As a member of the TOC, you may receive requests to speak at conferences, community days or events, or even on podcasts and webcasts and contribute to or participate in panels, blogs, and articles. It is important that you are aware of the capacity that you are participating in these activities in.  Depending on the activity, particularly for conferences and community days or events facilitated by the CNCF, support may be available to allow a TOC member to attend and speak. If you are approached to do this for a CNCF facilitated event and interested, reach out to CNCF Staff.

It is important that you conduct yourself in a manner expected of a Technical Leader that aligns, exemplifies, and promotes the [TOC Leadership Principles](../PRINCIPLES.md).
